--
-- ****************************************
-- top_dlx 
--    contains : everything!
-- ****************************************
--
library Synopsys, IEEE;
use Synopsys.attributes.all;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_signed.all;

entity TOP_DLX is
   port(
         PC            : out std_logic_vector(31 downto 0);
         reset         :  in std_logic;
         phi2          :  in std_logic
       );
end TOP_DLX;

architecture behavioral of TOP_DLX is 

   component IFETCH
   port(
         LInstruction : out std_logic_vector(31 downto 0);
         Instout      : out std_logic_vector(31 downto 0);
         do_Branch    :  in std_logic;
         phi2         :  in std_logic;
         reset        :  in std_logic;
         PCout        : out std_logic_vector(31 downto 0);
         LPCAdd       :  in std_logic_vector(31 downto 0);
         LNewPC       : out std_logic_vector(31 downto 0);
         Stallvect    :  in std_logic_vector(3 downto 0)
       );
   end component;

   component EXECUTE
   port(
         Lrr1d_bus     :  in std_logic_vector(31 downto 0);
         Lrr2d_bus     :  in std_logic_vector(31 downto 0);
        LLrr2d_bus     : out std_logic_vector(31 downto 0);
         Lextend       :  in std_logic_vector(31 downto 0);
         Lwra_bus      :  in std_logic_vector( 4 downto 0);
        LLwra_bus      : out std_logic_vector( 4 downto 0);
         LALUOut       : out std_logic_vector(31 downto 0);
         SysCont       : out std_logic_vector( 7 downto 0);
        LSysCont       : out std_logic_vector( 7 downto 0);
        LZero          :  in std_logic;
        LEql           :  in std_logic;
         reset         :  in std_logic;
         phi2          :  in std_logic;
        Lmux_out       :  in std_logic_vector(31 downto 0);
        Lra_bus_out    :  in std_logic_vector(31 downto 0);

	LLInstruction  :   in std_logic_vector(31 downto 0);
	LLLInstruction :  out std_logic_vector(31 downto 0);
-- Hazardvect
        Hazardvect     :   in std_logic_vector(11 downto 0);
        Stallvect      :   in std_logic_vector( 3 downto 0);
        ForceMuxVect   :   in std_logic_vector( 3 downto 0)
       );

   end component;

   component IDECODE
   port(
          LInstruction     :  in std_logic_vector(31 downto 0);
          Lrr1d_bus        : out std_logic_vector(31 downto 0);
          Lrr2d_bus        : out std_logic_vector(31 downto 0);
          RegWrite         :  in std_logic;
          LExtend          : out std_logic_vector(31 downto 0);
--        phi1             :  in std_logic;
          phi2             :  in std_logic;
          reset            :  in std_logic;
          Lwra_bus         : out std_logic_vector(4 downto 0);
          Lwra2_bus        : out std_logic_vector(4 downto 0);
          PCadd            :  in std_logic_vector(31 downto 0);
          LPCadd           : out std_logic_vector(31 downto 0);
          LZero            : out std_logic;
          wraddress        :  in std_logic_vector(4 downto 0);
          wrd_bus          :  in std_logic_vector(31 downto 0);
          Lrr1a_bus        : out std_logic_vector(4 downto 0);
          Lrr2a_bus        : out std_logic_vector(4 downto 0);
-- BEQ/BNE 
          Leql             : out std_logic;
	  LLInstruction    : out std_logic_vector(31 downto 0);
          Stallvect        :  in std_logic_vector( 3 downto 0)
       );
   end component;

   component DMEMORY
   port(
        rd_bus       :  out std_logic_vector(31 downto 0);
        LALUOut      :   in std_logic_vector(31 downto 0);
        LLrr2d_bus   :   in std_logic_vector(31 downto 0);
        MemRead      :   in std_logic;
        Memwrite     :   in std_logic;
        MemtoReg     :   in std_logic;
--      phi1         :   in std_logic;
        phi2         :   in std_logic;
        reset        :   in std_logic;
        LLwra_bus    :   in std_logic_vector(4 downto 0);
        LLLwra_bus   :  out std_logic_vector(4 downto 0);
        LSysCont     :   in std_logic_vector(7 downto 0);
        LLSysCont    :  out std_logic_vector(7 downto 0);
        Lmux_out     :  out std_logic_vector(31 downto 0);
        Lra_bus_out  :  out std_logic_vector(31 downto 0);

	LLLInstruction   :  in std_logic_vector(31 downto 0);
	LLLLInstruction  : out std_logic_vector(31 downto 0);
        Hazardvect       :  in std_logic_vector(11 downto 0)
       );
   end component;

   component HAZARD
   port(
        Instruction     :  in std_logic_vector(31 downto 0);
	LInstruction    :  in std_logic_vector(31 downto 0);
	LLInstruction   :  in std_logic_vector(31 downto 0);
	LLLInstruction  :  in std_logic_vector(31 downto 0);
	LLLLInstruction :  in std_logic_vector(31 downto 0);
--      phi1	        :  in std_logic;
	phi2 		:  in std_logic;
	reset	 	:  in std_logic;
	Hazardvect      : out std_logic_vector(11 downto 0);
        Stallvect       : out std_logic_vector( 3 downto 0);
        ForceMuxVect    : out std_logic_vector( 3 downto 0)
	);
   end component;


-- put signals here
-- 8bit
   signal Instru12     : std_logic_vector(31 downto 0);
   signal PC12         : std_logic_vector(31 downto 0);
   signal LPCAdd31     : std_logic_vector(31 downto 0);
   signal rr1d23       : std_logic_vector(31 downto 0);
   signal rr2d23       : std_logic_vector(31 downto 0);
   signal lrr2d34      : std_logic_vector(31 downto 0);
   signal extend23     : std_logic_vector(31 downto 0);
   signal wra_bus23    : std_logic_vector( 4 downto 0);
   signal lwra_bus34   : std_logic_vector( 4 downto 0);
-- signal lop23        : std_logic_vector( 5 downto 0);
-- signal lopx23       : std_logic_vector( 5 downto 0);
   signal llwra_bus42  : std_logic_vector( 4 downto 0);
   signal aluout34     : std_logic_vector(31 downto 0);
   signal rdbus52      : std_logic_vector(31 downto 0);
   signal SysCont34    : std_logic_vector(7 downto 0);
-- do_branch
   signal SysCont3     : std_logic_vector(7 downto 0);
   signal SysCont45    : std_logic_vector(7 downto 0);
   signal lzero23      : std_logic;
   signal LEql23       : std_logic;

-- junk signals (use for haz detect)
   signal lwra2_bus    : std_logic_vector(4 downto 0);
   signal lrr1a_bus    : std_logic_vector(4 downto 0);
   signal lrr2a_bus    : std_logic_vector(4 downto 0);

-- junk for (hazards) (jat)
   signal lmuxout53    : std_logic_vector(31 downto 0);
   signal lrabusout53  : std_logic_vector(31 downto 0);

-- BNE/BEQ
   signal Leql         : std_logic;

   signal Instru23     : std_logic_vector(31 downto 0);
   signal Instru34     : std_logic_vector(31 downto 0);
   signal Instru45     : std_logic_vector(31 downto 0);
   signal Hazardvect   : std_logic_vector(11 downto 0);
   signal Stallvect    : std_logic_vector( 3 downto 0);
   signal ForceMuxVect : std_logic_vector( 3 downto 0);
   signal Instout      : std_logic_vector(31 downto 0);

BEGIN

   IFETCH1: IFETCH
   port map(
         LInstruction => Instru12,
         Instout      => Instout,
         do_Branch    => SysCont3(1),
--       phi1         => phi1,
         phi2         => phi2,
         reset        => reset,
         PCout        => PC,
         LPCAdd       => LPCAdd31,
         LNewPC       => PC12,
         Stallvect    => Stallvect
           );

   EXECUTE1: EXECUTE
   port map(
         Lrr1d_bus  => rr1d23,
         Lrr2d_bus  => rr2d23,
        LLrr2d_bus  => lrr2d34,
         Lextend    => extend23,
         Lwra_bus   => wra_bus23,
        LLwra_bus   => lwra_bus34,
--       Lop        => lop23,
--       Lopx       => lopx23,
         LALUOut    => aluout34,
         SysCont    => SysCont3,
        LSysCont    => SysCont34,
        LZero       => lzero23,
        LEql        => LEql23,
         reset      => reset,
--       phi1       => phi1,
         phi2       => phi2,
         Lmux_out   => lmuxout53,
        Lra_bus_out => lrabusout53,

      LLInstruction => Instru23,
     LLLInstruction => Instru34,
        Hazardvect  => Hazardvect,
        Stallvect   => Stallvect,
      ForceMuxVect  => ForceMuxVect 
           );

   IDECODE1: IDECODE
   port map(
          LInstruction => Instru12,
          Lrr1d_bus    => rr1d23,
          Lrr2d_bus    => rr2d23,
          RegWrite     => SysCont45(0),
          LExtend      => extend23,
--        phi1         => phi1,
          phi2         => phi2,
          reset        => reset,
          Lwra_bus     => wra_bus23,
-- haz sig
          Lwra2_bus    => lwra2_bus,
          PCadd        => PC12,
          LPCadd       => LPCAdd31,
          LZero        => lzero23,
          wraddress    => llwra_bus42,
          wrd_bus      => rdbus52,
-- haz sig
          Lrr1a_bus    => lrr1a_bus,
-- haz sig
          Lrr2a_bus    => lrr2a_bus,
--        Lop          => lop23,
--        Lopx         => lopx23,
-- BEQ/BNE
          Leql         => Leql23,
	LLInstruction => Instru23,
          Stallvect   => Stallvect
           );

   DMEMORY1: DMEMORY
   port map(
        rd_bus        => rdbus52,
        LALUOut       => aluout34,
        LLrr2d_bus    => lrr2d34,
        MemRead       => SysCont45(3),
        Memwrite      => SysCont34(2),
        MemtoReg      => SysCont45(4),
--      phi1          => phi1,
        phi2          => phi2,
        reset         => reset,
        LLwra_bus     => lwra_bus34,
        LLLwra_bus    => llwra_bus42,
        LSysCont      => SysCont34,
       LLSysCont      => SysCont45,
        Lmux_out      => lmuxout53, 
        Lra_bus_out   => lrabusout53,
	LLLInstruction  => Instru34,
	LLLLInstruction => Instru45,
         Hazardvect     => Hazardvect
           );

   HAZARD1: HAZARD
   port map(
        Instruction     => Instout,
	LInstruction    => Instru12,
	LLInstruction   => Instru23,
	LLLInstruction  => Instru34,
	LLLLInstruction => Instru45,
--      phi1            => phi1,
	phi2            => phi2,
	reset	        => reset,
	Hazardvect      => Hazardvect,
        Stallvect       => Stallvect,
      ForceMuxVect      => ForceMuxVect 
	);

-- ***************************************************************
--   dobra31 <= SysCont(6);

-- ***************************************************************

END behavioral;

configuration DLX1 of TOP_DLX is
for behavioral
  for IFETCH1: IFETCH
     use entity work.IFETCH(behavioral);
  end for;
  for EXECUTE1: EXECUTE
     use entity work.EXECUTE(behavioral);
  end for;
  for IDECODE1: IDECODE
      use entity work.IDECODE(behavioral);    
  end for;
  for DMEMORY1: DMEMORY
      use entity work.DMEMORY(behavioral);    
  end for;
  for HAZARD1: HAZARD
      use entity work.HAZARD(behavioral);
  end for;

end for;
end DLX1;

